// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.aiplatform.v1;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Cloud.AIPlatform.V1";
option go_package = "google.golang.org/genproto/googleapis/cloud/aiplatform/v1;aiplatform";
option java_multiple_files = true;
option java_outer_classname = "PipelineStateProto";
option java_package = "com.google.cloud.aiplatform.v1";
option php_namespace = "Google\\Cloud\\AIPlatform\\V1";
option ruby_package = "Google::Cloud::AIPlatform::V1";

// Describes the state of a pipeline.
enum PipelineState {
  // The pipeline state is unspecified.
  PIPELINE_STATE_UNSPECIFIED = 0;

  // The pipeline has been created or resumed, and processing has not yet
  // begun.
  PIPELINE_STATE_QUEUED = 1;

  // The service is preparing to run the pipeline.
  PIPELINE_STATE_PENDING = 2;

  // The pipeline is in progress.
  PIPELINE_STATE_RUNNING = 3;

  // The pipeline completed successfully.
  PIPELINE_STATE_SUCCEEDED = 4;

  // The pipeline failed.
  PIPELINE_STATE_FAILED = 5;

  // The pipeline is being cancelled. From this state, the pipeline may only go
  // to either PIPELINE_STATE_SUCCEEDED, PIPELINE_STATE_FAILED or
  // PIPELINE_STATE_CANCELLED.
  PIPELINE_STATE_CANCELLING = 6;

  // The pipeline has been cancelled.
  PIPELINE_STATE_CANCELLED = 7;

  // The pipeline has been stopped, and can be resumed.
  PIPELINE_STATE_PAUSED = 8;
}
