// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.maps.routes.v1;

option cc_enable_arenas = true;
option csharp_namespace = "Google.Maps.Routes.V1";
option go_package = "google.golang.org/genproto/googleapis/maps/routes/v1;routes";
option java_multiple_files = true;
option java_outer_classname = "TollPassesProto";
option java_package = "com.google.maps.routes.v1";
option objc_class_prefix = "GMRS";
option php_namespace = "Google\\Maps\\Routes\\V1";

// List of toll passes around the world that we support.
enum TollPass {
  // Not used. If this value is used, then the request fails.
  TOLL_PASS_UNSPECIFIED = 0;

  // Australia-wide toll pass.
  // See additional details at https://www.linkt.com.au/.
  AU_LINKT = 2;

  // Argentina toll pass. See additional details at https://telepase.com.ar
  AR_TELEPASE = 3;

  // Brazil toll pass. See additional details at https://conectcar.com.
  BR_CONECTCAR = 7;

  // Brazil toll pass. See additional details at https://movemais.com.
  BR_MOVE_MAIS = 8;

  // Brazil toll pass. See additional details at https://www.semparar.com.br.
  BR_SEM_PARAR = 9;

  // Brazil toll pass. See additional details at https://taggy.com.br.
  BR_TAGGY = 10;

  // Brazil toll pass. See additional details at
  // https://veloe.com.br/site/onde-usar.
  BR_VELOE = 11;

  // Mexico toll pass.
  MX_TAG_IAVE = 12;

  // Mexico toll pass company. One of many operating in Mexico City. See
  // additional details at https://www.televia.com.mx.
  MX_TAG_TELEVIA = 13;

  // Mexico toll pass. See additional details at
  // https://www.viapass.com.mx/viapass/web_home.aspx.
  MX_VIAPASS = 14;

  // State pass of California, United States. Passes vary between Standard,
  // Flex, and Clean Air. Flex and Clean Air have settings for carpool. See
  // additional details at https://www.bayareafastrak.org/en/home/index.shtml.
  US_CA_FASTRAK = 4;

  // State pass of Illinois, United States. See additional details at
  // https://www.illinoistollway.com/about-ipass.
  US_IL_IPASS = 5;

  // State pass of Massachusetts, United States. See additional details at
  // https://www.mass.gov/ezdrivema.
  US_MA_EZPASSMA = 6;

  // State pass of New York, United States. See additional details at
  // https://www.e-zpassny.com.
  US_NY_EZPASSNY = 15;

  // State pass of the Washington state, United States.
  US_WA_GOOD_TO_GO = 1;
}
