// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v7.errors;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V7.Errors";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v7/errors;errors";
option java_multiple_files = true;
option java_outer_classname = "AssetErrorProto";
option java_package = "com.google.ads.googleads.v7.errors";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V7\\Errors";
option ruby_package = "Google::Ads::GoogleAds::V7::Errors";

// Proto file describing asset errors.

// Container for enum describing possible asset errors.
message AssetErrorEnum {
  // Enum describing possible asset errors.
  enum AssetError {
    // Enum unspecified.
    UNSPECIFIED = 0;

    // The received error code is not known in this version.
    UNKNOWN = 1;

    // The customer is not is not on the allow-list for this asset type.
    CUSTOMER_NOT_ON_ALLOWLIST_FOR_ASSET_TYPE = 13;

    // Assets are duplicated across operations.
    DUPLICATE_ASSET = 3;

    // The asset name is duplicated, either across operations or with an
    // existing asset.
    DUPLICATE_ASSET_NAME = 4;

    // The Asset.asset_data oneof is empty.
    ASSET_DATA_IS_MISSING = 5;

    // The asset has a name which is different from an existing duplicate that
    // represents the same content.
    CANNOT_MODIFY_ASSET_NAME = 6;

    // The field cannot be set for this asset type.
    FIELD_INCOMPATIBLE_WITH_ASSET_TYPE = 7;

    // Call to action must come from the list of supported values.
    INVALID_CALL_TO_ACTION_TEXT = 8;

    // A lead form asset is created with an invalid combination of input fields.
    LEAD_FORM_INVALID_FIELDS_COMBINATION = 9;

    // Lead forms require that the Terms of Service have been agreed to before
    // mutates can be executed.
    LEAD_FORM_MISSING_AGREEMENT = 10;

    // Asset status is invalid in this operation.
    INVALID_ASSET_STATUS = 11;

    // The field cannot be modified by this asset type.
    FIELD_CANNOT_BE_MODIFIED_FOR_ASSET_TYPE = 12;

    // Ad schedules for the same asset cannot overlap.
    SCHEDULES_CANNOT_OVERLAP = 14;

    // Cannot set both percent off and money amount off fields of promotion
    // asset.
    PROMOTION_CANNOT_SET_PERCENT_OFF_AND_MONEY_AMOUNT_OFF = 15;

    // Cannot set both promotion code and orders over amount fields of promotion
    // asset.
    PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT = 16;

    // The field has too many decimal places specified.
    TOO_MANY_DECIMAL_PLACES_SPECIFIED = 17;

    // Duplicate assets across operations, which have identical Asset.asset_data
    // oneof, cannot have different asset level fields for asset types which are
    // deduped.
    DUPLICATE_ASSETS_WITH_DIFFERENT_FIELD_VALUE = 18;
  }


}
