// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.dialogflow.v2beta1;

import "google/api/field_behavior.proto";
import "google/api/annotations.proto";

option cc_enable_arenas = true;
option csharp_namespace = "Google.Cloud.Dialogflow.V2beta1";
option go_package = "google.golang.org/genproto/googleapis/cloud/dialogflow/v2beta1;dialogflow";
option java_multiple_files = true;
option java_outer_classname = "GcsProto";
option java_package = "com.google.cloud.dialogflow.v2beta1";
option objc_class_prefix = "DF";

// Google Cloud Storage locations for the inputs.
message GcsSources {
  // Required. Google Cloud Storage URIs for the inputs. A URI is of the
  // form:
  //   gs://bucket/object-prefix-or-name
  // Whether a prefix or name is used depends on the use case.
  repeated string uris = 2 [(google.api.field_behavior) = REQUIRED];
}

// Google Cloud Storage location for single input.
message GcsSource {
  // Required. The Google Cloud Storage URIs for the inputs. A URI is of the
  // form:
  //   gs://bucket/object-prefix-or-name
  // Whether a prefix or name is used depends on the use case.
  string uri = 1;
}
