// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v7.enums;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V7.Enums";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v7/enums;enums";
option java_multiple_files = true;
option java_outer_classname = "AssetFieldTypeProto";
option java_package = "com.google.ads.googleads.v7.enums";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V7\\Enums";
option ruby_package = "Google::Ads::GoogleAds::V7::Enums";

// Proto file describing asset type.

// Container for enum describing the possible placements of an asset.
message AssetFieldTypeEnum {
  // Enum describing the possible placements of an asset.
  enum AssetFieldType {
    // Not specified.
    UNSPECIFIED = 0;

    // Used for return value only. Represents value unknown in this version.
    UNKNOWN = 1;

    // The asset is linked for use as a headline.
    HEADLINE = 2;

    // The asset is linked for use as a description.
    DESCRIPTION = 3;

    // The asset is linked for use as mandatory ad text.
    MANDATORY_AD_TEXT = 4;

    // The asset is linked for use as a marketing image.
    MARKETING_IMAGE = 5;

    // The asset is linked for use as a media bundle.
    MEDIA_BUNDLE = 6;

    // The asset is linked for use as a YouTube video.
    YOUTUBE_VIDEO = 7;

    // The asset is linked to indicate that a hotels campaign is "Book on
    // Google" enabled.
    BOOK_ON_GOOGLE = 8;

    // The asset is linked for use as a Lead Form extension.
    LEAD_FORM = 9;

    // The asset is linked for use as a Promotion extension.
    PROMOTION = 10;

    // The asset is linked for use as a Callout extension.
    CALLOUT = 11;

    // The asset is linked for use as a Structured Snippet extension.
    STRUCTURED_SNIPPET = 12;

    // The asset is linked for use as a Sitelink extension.
    SITELINK = 13;
  }


}
