// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.aiplatform.v1beta1.schema;

import "google/api/field_behavior.proto";
import "google/protobuf/duration.proto";
import "google/api/annotations.proto";

option go_package = "google.golang.org/genproto/googleapis/cloud/aiplatform/v1beta1/schema;schema";
option java_multiple_files = true;
option java_outer_classname = "DataItemPayloadProto";
option java_package = "com.google.cloud.aiplatform.v1beta1.schema";

// Payload of Image DataItem.
message ImageDataItem {
  // Required. Google Cloud Storage URI points to the original image in user's bucket.
  // The image is up to 30MB in size.
  string gcs_uri = 1 [(google.api.field_behavior) = REQUIRED];

  // Output only. The mime type of the content of the image. Only the images in below listed
  // mime types are supported.
  // - image/jpeg
  // - image/gif
  // - image/png
  // - image/webp
  // - image/bmp
  // - image/tiff
  // - image/vnd.microsoft.icon
  string mime_type = 2 [(google.api.field_behavior) = OUTPUT_ONLY];
}

// Payload of Video DataItem.
message VideoDataItem {
  // Required. Google Cloud Storage URI points to the original video in user's bucket.
  // The video is up to 50 GB in size and up to 3 hour in duration.
  string gcs_uri = 1 [(google.api.field_behavior) = REQUIRED];

  // Output only. The mime type of the content of the video. Only the videos in below listed
  // mime types are supported.
  // Supported mime_type:
  // - video/mp4
  // - video/avi
  // - video/quicktime
  string mime_type = 2 [(google.api.field_behavior) = OUTPUT_ONLY];
}

// Payload of Text DataItem.
message TextDataItem {
  // Output only. Google Cloud Storage URI points to the original text in user's bucket.
  // The text file is up to 10MB in size.
  string gcs_uri = 1 [(google.api.field_behavior) = OUTPUT_ONLY];
}
