// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v6.resources;

import "google/ads/googleads/v6/common/simulation.proto";
import "google/ads/googleads/v6/enums/simulation_modification_method.proto";
import "google/ads/googleads/v6/enums/simulation_type.proto";
import "google/api/field_behavior.proto";
import "google/api/resource.proto";
import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V6.Resources";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v6/resources;resources";
option java_multiple_files = true;
option java_outer_classname = "CampaignCriterionSimulationProto";
option java_package = "com.google.ads.googleads.v6.resources";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V6\\Resources";
option ruby_package = "Google::Ads::GoogleAds::V6::Resources";

// Proto file describing the campaign criterion simulation resource.

// A campaign criterion simulation. Supported combinations of advertising
// channel type, criterion ids, simulation type and simulation modification
// method is detailed below respectively.
//
// 1. SEARCH - 30000,30001,30002 - BID_MODIFIER - UNIFORM
// 2. SHOPPING - 30000,30001,30002 - BID_MODIFIER - UNIFORM
// 3. DISPLAY - 30001 - BID_MODIFIER - UNIFORM
message CampaignCriterionSimulation {
  option (google.api.resource) = {
    type: "googleads.googleapis.com/CampaignCriterionSimulation"
    pattern: "customers/{customer_id}/campaignCriterionSimulations/{campaign_id}~{criterion_id}~{type}~{modification_method}~{start_date}~{end_date}"
  };

  // Output only. The resource name of the campaign criterion simulation.
  // Campaign criterion simulation resource names have the form:
  //
  // `customers/{customer_id}/campaignCriterionSimulations/{campaign_id}~{criterion_id}~{type}~{modification_method}~{start_date}~{end_date}`
  string resource_name = 1 [
    (google.api.field_behavior) = OUTPUT_ONLY,
    (google.api.resource_reference) = {
      type: "googleads.googleapis.com/CampaignCriterionSimulation"
    }
  ];

  // Output only. Campaign ID of the simulation.
  optional int64 campaign_id = 9 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. Criterion ID of the simulation.
  optional int64 criterion_id = 10 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. The field that the simulation modifies.
  google.ads.googleads.v6.enums.SimulationTypeEnum.SimulationType type = 4 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. How the simulation modifies the field.
  google.ads.googleads.v6.enums.SimulationModificationMethodEnum.SimulationModificationMethod modification_method = 5 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. First day on which the simulation is based, in YYYY-MM-DD format.
  optional string start_date = 11 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. Last day on which the simulation is based, in YYYY-MM-DD format.
  optional string end_date = 12 [(google.api.field_behavior) = OUTPUT_ONLY];

  // List of simulation points.
  oneof point_list {
    // Output only. Simulation points if the simulation type is BID_MODIFIER.
    google.ads.googleads.v6.common.BidModifierSimulationPointList bid_modifier_point_list = 8 [(google.api.field_behavior) = OUTPUT_ONLY];
  }
}
