// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.aiplatform.v1beta1;

import "google/protobuf/duration.proto";
import "google/api/annotations.proto";

option csharp_namespace = "Google.Cloud.AIPlatform.V1Beta1";
option go_package = "google.golang.org/genproto/googleapis/cloud/aiplatform/v1beta1;aiplatform";
option java_multiple_files = true;
option java_outer_classname = "FeaturestoreMonitoringProto";
option java_package = "com.google.cloud.aiplatform.v1beta1";
option php_namespace = "Google\\Cloud\\AIPlatform\\V1beta1";
option ruby_package = "Google::Cloud::AIPlatform::V1beta1";

// Configuration of how features in Featurestore are monitored.
message FeaturestoreMonitoringConfig {
  // Configuration of the Featurestore's Snapshot Analysis Based Monitoring.
  // This type of analysis generates statistics for each Feature based on a
  // snapshot of the latest feature value of each entities every
  // monitoring_interval.
  message SnapshotAnalysis {
    // The monitoring schedule for snapshot analysis.
    // For EntityType-level config:
    //   unset / disabled = true indicates disabled by
    //   default for Features under it; otherwise by default enable snapshot
    //   analysis monitoring with monitoring_interval for Features under it.
    // Feature-level config:
    //   disabled = true indicates disabled regardless of the EntityType-level
    //   config; unset monitoring_interval indicates going with EntityType-level
    //   config; otherwise run snapshot analysis monitoring with
    //   monitoring_interval regardless of the EntityType-level config.
    // Explicitly Disable the snapshot analysis based monitoring.
    bool disabled = 1;

    // Configuration of the snapshot analysis based monitoring pipeline
    // running interval. The value is rolled up to full day.
    google.protobuf.Duration monitoring_interval = 2;

    // Configuration of the snapshot analysis based monitoring pipeline
    // running interval. The value indicates number of days.
    // If both
    // [FeaturestoreMonitoringConfig.SnapshotAnalysis.monitoring_interval_days][google.cloud.aiplatform.v1beta1.FeaturestoreMonitoringConfig.SnapshotAnalysis.monitoring_interval_days]
    // and [FeaturestoreMonitoringConfig.SnapshotAnalysis.monitoring_interval][google.cloud.aiplatform.v1beta1.FeaturestoreMonitoringConfig.SnapshotAnalysis.monitoring_interval]
    // are set when creating/updating EntityTypes/Features,
    // [FeaturestoreMonitoringConfig.SnapshotAnalysis.monitoring_interval_days][google.cloud.aiplatform.v1beta1.FeaturestoreMonitoringConfig.SnapshotAnalysis.monitoring_interval_days]
    // will be used.
    int32 monitoring_interval_days = 3;
  }

  // The config for Snapshot Analysis Based Feature Monitoring.
  SnapshotAnalysis snapshot_analysis = 1;
}
