// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.retail.v2alpha;

import "google/api/annotations.proto";
import "google/api/client.proto";
import "google/api/field_behavior.proto";
import "google/api/resource.proto";
import "google/cloud/retail/v2alpha/catalog.proto";
import "google/protobuf/empty.proto";
import "google/protobuf/field_mask.proto";
import "google/protobuf/timestamp.proto";

option csharp_namespace = "Google.Cloud.Retail.V2Alpha";
option go_package = "google.golang.org/genproto/googleapis/cloud/retail/v2alpha;retail";
option java_multiple_files = true;
option java_outer_classname = "CatalogServiceProto";
option java_package = "com.google.cloud.retail.v2alpha";
option objc_class_prefix = "RETAIL";
option php_namespace = "Google\\Cloud\\Retail\\V2alpha";
option ruby_package = "Google::Cloud::Retail::V2alpha";

// Service for managing catalog configuration.
service CatalogService {
  option (google.api.default_host) = "retail.googleapis.com";
  option (google.api.oauth_scopes) =
      "https://www.googleapis.com/auth/cloud-platform";

  // Lists all the [Catalog][google.cloud.retail.v2alpha.Catalog]s associated
  // with the project.
  rpc ListCatalogs(ListCatalogsRequest) returns (ListCatalogsResponse) {
    option (google.api.http) = {
      get: "/v2alpha/{parent=projects/*/locations/*}/catalogs"
    };
    option (google.api.method_signature) = "parent";
  }

  // Updates the [Catalog][google.cloud.retail.v2alpha.Catalog]s.
  rpc UpdateCatalog(UpdateCatalogRequest) returns (Catalog) {
    option (google.api.http) = {
      patch: "/v2alpha/{catalog.name=projects/*/locations/*/catalogs/*}"
      body: "catalog"
    };
    option (google.api.method_signature) = "catalog,update_mask";
  }

  // Set a specified branch id as default branch. API methods such as
  // [SearchService.Search][google.cloud.retail.v2alpha.SearchService.Search],
  // [ProductService.GetProduct][google.cloud.retail.v2alpha.ProductService.GetProduct],
  // [ProductService.ListProducts][google.cloud.retail.v2alpha.ProductService.ListProducts]
  // will treat requests using "default_branch" to the actual branch id set as
  // default.
  //
  // For example, if `projects/*/locations/*/catalogs/*/branches/1` is set as
  // default, setting
  // [SearchRequest.branch][google.cloud.retail.v2alpha.SearchRequest.branch] to
  // `projects/*/locations/*/catalogs/*/branches/default_branch` is equivalent
  // to setting
  // [SearchRequest.branch][google.cloud.retail.v2alpha.SearchRequest.branch] to
  // `projects/*/locations/*/catalogs/*/branches/1`.
  //
  // Using multiple branches can be useful when developers would like
  // to have a staging branch to test and verify for future usage. When it
  // becomes ready, developers switch on the staging branch using this API while
  // keeping using `projects/*/locations/*/catalogs/*/branches/default_branch`
  // as [SearchRequest.branch][google.cloud.retail.v2alpha.SearchRequest.branch]
  // to route the traffic to this staging branch.
  //
  // CAUTION: If you have live predict/search traffic, switching the default
  // branch could potentially cause outages if the ID space of the new branch is
  // very different from the old one.
  //
  // More specifically:
  //
  // * PredictionService will only return product IDs from branch {newBranch}.
  // * SearchService will only return product IDs from branch {newBranch}
  //   (if branch is not explicitly set).
  // * UserEventService will only join events with products from branch
  //   {newBranch}.
  //
  // This feature is only available for users who have Retail Search enabled.
  // Contact Retail Support (retail-search-support@google.com) if you are
  // interested in using Retail Search.
  rpc SetDefaultBranch(SetDefaultBranchRequest)
      returns (google.protobuf.Empty) {
    option (google.api.http) = {
      post: "/v2alpha/{catalog=projects/*/locations/*/catalogs/*}:setDefaultBranch"
      body: "*"
    };
    option (google.api.method_signature) = "catalog";
  }

  // Get which branch is currently default branch set by
  // [CatalogService.SetDefaultBranch][google.cloud.retail.v2alpha.CatalogService.SetDefaultBranch]
  // method under a specified parent catalog.
  //
  // This feature is only available for users who have Retail Search enabled.
  // Contact Retail Support (retail-search-support@google.com) if you are
  // interested in using Retail Search.
  rpc GetDefaultBranch(GetDefaultBranchRequest)
      returns (GetDefaultBranchResponse) {
    option (google.api.http) = {
      get: "/v2alpha/{catalog=projects/*/locations/*/catalogs/*}:getDefaultBranch"
    };
    option (google.api.method_signature) = "catalog";
  }
}

// Request for
// [CatalogService.ListCatalogs][google.cloud.retail.v2alpha.CatalogService.ListCatalogs]
// method.
message ListCatalogsRequest {
  // Required. The account resource name with an associated location.
  //
  // If the caller does not have permission to list
  // [Catalog][google.cloud.retail.v2alpha.Catalog]s under this location,
  // regardless of whether or not this location exists, a PERMISSION_DENIED
  // error is returned.
  string parent = 1 [
    (google.api.field_behavior) = REQUIRED,
    (google.api.resource_reference) = {
      type: "locations.googleapis.com/Location"
    }
  ];

  // Maximum number of [Catalog][google.cloud.retail.v2alpha.Catalog]s to
  // return. If unspecified, defaults to 50. The maximum allowed value is 1000.
  // Values above 1000 will be coerced to 1000.
  //
  // If this field is negative, an INVALID_ARGUMENT is returned.
  int32 page_size = 2;

  // A page token
  // [ListCatalogsResponse.next_page_token][google.cloud.retail.v2alpha.ListCatalogsResponse.next_page_token],
  // received from a previous
  // [CatalogService.ListCatalogs][google.cloud.retail.v2alpha.CatalogService.ListCatalogs]
  // call. Provide this to retrieve the subsequent page.
  //
  // When paginating, all other parameters provided to
  // [CatalogService.ListCatalogs][google.cloud.retail.v2alpha.CatalogService.ListCatalogs]
  // must match the call that provided the page token. Otherwise, an
  // INVALID_ARGUMENT error is returned.
  string page_token = 3;
}

// Response for
// [CatalogService.ListCatalogs][google.cloud.retail.v2alpha.CatalogService.ListCatalogs]
// method.
message ListCatalogsResponse {
  // All the customer's [Catalog][google.cloud.retail.v2alpha.Catalog]s.
  repeated Catalog catalogs = 1;

  // A token that can be sent as
  // [ListCatalogsRequest.page_token][google.cloud.retail.v2alpha.ListCatalogsRequest.page_token]
  // to retrieve the next page. If this field is omitted, there are no
  // subsequent pages.
  string next_page_token = 2;
}

// Request for
// [CatalogService.UpdateCatalog][google.cloud.retail.v2alpha.CatalogService.UpdateCatalog]
// method.
message UpdateCatalogRequest {
  // Required. The [Catalog][google.cloud.retail.v2alpha.Catalog] to update.
  //
  // If the caller does not have permission to update the
  // [Catalog][google.cloud.retail.v2alpha.Catalog], regardless of whether or
  // not it exists, a PERMISSION_DENIED error is returned.
  //
  // If the [Catalog][google.cloud.retail.v2alpha.Catalog] to update does not
  // exist, a NOT_FOUND error is returned.
  Catalog catalog = 1 [(google.api.field_behavior) = REQUIRED];

  // Indicates which fields in the provided
  // [Catalog][google.cloud.retail.v2alpha.Catalog] to update.
  //
  // If an unsupported or unknown field is provided, an INVALID_ARGUMENT error
  // is returned.
  google.protobuf.FieldMask update_mask = 2;
}

// Request message to set a specified branch as new default_branch.
message SetDefaultBranchRequest {
  // Full resource name of the catalog, such as
  // `projects/*/locations/global/catalogs/default_catalog`.
  string catalog = 1 [
    (google.api.resource_reference) = { type: "retail.googleapis.com/Catalog" }
  ];

  // The final component of the resource name of a branch.
  //
  // This field must be one of "0", "1" or "2". Otherwise, an INVALID_ARGUMENT
  // error is returned.
  string branch_id = 2 [
    (google.api.resource_reference) = { type: "retail.googleapis.com/Branch" }
  ];

  // Some note on this request, this can be retrieved by
  // [CatalogService.GetDefaultBranch][google.cloud.retail.v2alpha.CatalogService.GetDefaultBranch]
  // before next valid default branch set occurs.
  //
  // This field must be a UTF-8 encoded string with a length limit of 1,000
  // characters. Otherwise, an INVALID_ARGUMENT error is returned.
  string note = 3;
}

// Request message to show which branch is currently the default branch.
message GetDefaultBranchRequest {
  // The parent catalog resource name, such as
  // `projects/*/locations/global/catalogs/default_catalog`.
  string catalog = 1 [
    (google.api.resource_reference) = { type: "retail.googleapis.com/Catalog" }
  ];
}

// Response message of
// [CatalogService.GetDefaultBranch][google.cloud.retail.v2alpha.CatalogService.GetDefaultBranch].
message GetDefaultBranchResponse {
  // Full resource name of the branch id currently set as default branch.
  string branch = 1 [
    (google.api.resource_reference) = { type: "retail.googleapis.com/Branch" }
  ];

  // The time when this branch is set to default.
  google.protobuf.Timestamp set_time = 2;

  // This corresponds to
  // [SetDefaultBranchRequest.note][google.cloud.retail.v2alpha.SetDefaultBranchRequest.note]
  // field, when this branch was set as default.
  string note = 3;
}
