// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.sql.v1beta4;

import "google/api/annotations.proto";
import "google/api/client.proto";

option go_package = "google.golang.org/genproto/googleapis/cloud/sql/v1beta4;sql";
option java_multiple_files = true;
option java_outer_classname = "CloudSqlTiersProto";
option java_package = "com.google.cloud.sql.v1beta4";

// LINT: LEGACY_NAMES

// Service for providing machine types (tiers) for Cloud SQL.
service SqlTiersService {
  option (google.api.default_host) = "sqladmin.googleapis.com";
  option (google.api.oauth_scopes) =
      "https://www.googleapis.com/auth/cloud-platform,"
      "https://www.googleapis.com/auth/sqlservice.admin";

  // Lists all available machine types (tiers) for Cloud SQL, for example,
  // db-custom-1-3840. For related information, see <a
  // href="/sql/pricing">Pricing</a>.
  rpc List(SqlTiersListRequest) returns (TiersListResponse) {
    option (google.api.http) = {
      get: "/sql/v1beta4/projects/{project}/tiers"
    };
  }
}

message SqlTiersListRequest {
  // Project ID of the project for which to list tiers.
  string project = 1;
}

// Tiers list response.
message TiersListResponse {
  // This is always <b>sql#tiersList</b>.
  string kind = 1;

  // List of tiers.
  repeated Tier items = 2;
}

// A Google Cloud SQL service tier resource.
message Tier {
  // An identifier for the machine type, for example, db-custom-1-3840. For
  // related information, see <a href="/sql/pricing">Pricing</a>.
  string tier = 1;

  // The maximum RAM usage of this tier in bytes.
  int64 RAM = 2;

  // This is always <b>sql#tier</b>.
  string kind = 3;

  // The maximum disk size of this tier in bytes.
  int64 Disk_Quota = 4;

  // The applicable regions for this tier.
  repeated string region = 5;
}
