// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.aiplatform.v1beta1;

import "google/api/field_behavior.proto";
import "google/api/annotations.proto";

option csharp_namespace = "Google.Cloud.AIPlatform.V1Beta1";
option go_package = "google.golang.org/genproto/googleapis/cloud/aiplatform/v1beta1;aiplatform";
option java_multiple_files = true;
option java_outer_classname = "IoProto";
option java_package = "com.google.cloud.aiplatform.v1beta1";
option php_namespace = "Google\\Cloud\\AIPlatform\\V1beta1";
option ruby_package = "Google::Cloud::AIPlatform::V1beta1";

// The storage details for Avro input content.
message AvroSource {
  // Required. Google Cloud Storage location.
  GcsSource gcs_source = 1 [(google.api.field_behavior) = REQUIRED];
}

// The storage details for CSV input content.
message CsvSource {
  // Required. Google Cloud Storage location.
  GcsSource gcs_source = 1 [(google.api.field_behavior) = REQUIRED];
}

// The Google Cloud Storage location for the input content.
message GcsSource {
  // Required. Google Cloud Storage URI(-s) to the input file(s). May contain
  // wildcards. For more information on wildcards, see
  // https://cloud.google.com/storage/docs/gsutil/addlhelp/WildcardNames.
  repeated string uris = 1 [(google.api.field_behavior) = REQUIRED];
}

// The Google Cloud Storage location where the output is to be written to.
message GcsDestination {
  // Required. Google Cloud Storage URI to output directory. If the uri doesn't end with
  // '/', a '/' will be automatically appended. The directory is created if it
  // doesn't exist.
  string output_uri_prefix = 1 [(google.api.field_behavior) = REQUIRED];
}

// The BigQuery location for the input content.
message BigQuerySource {
  // Required. BigQuery URI to a table, up to 2000 characters long.
  // Accepted forms:
  //
  // *  BigQuery path. For example: `bq://projectId.bqDatasetId.bqTableId`.
  string input_uri = 1 [(google.api.field_behavior) = REQUIRED];
}

// The BigQuery location for the output content.
message BigQueryDestination {
  // Required. BigQuery URI to a project or table, up to 2000 characters long.
  //
  // When only the project is specified, the Dataset and Table is created.
  // When the full table reference is specified, the Dataset must exist and
  // table must not exist.
  //
  // Accepted forms:
  //
  // *  BigQuery path. For example:
  // `bq://projectId` or `bq://projectId.bqDatasetId` or
  // `bq://projectId.bqDatasetId.bqTableId`.
  string output_uri = 1 [(google.api.field_behavior) = REQUIRED];
}

// The storage details for CSV output content.
message CsvDestination {
  // Required. Google Cloud Storage location.
  GcsDestination gcs_destination = 1 [(google.api.field_behavior) = REQUIRED];
}

// The storage details for TFRecord output content.
message TFRecordDestination {
  // Required. Google Cloud Storage location.
  GcsDestination gcs_destination = 1 [(google.api.field_behavior) = REQUIRED];
}

// The Container Registry location for the container image.
message ContainerRegistryDestination {
  // Required. Container Registry URI of a container image.
  // Only Google Container Registry and Artifact Registry are supported now.
  // Accepted forms:
  //
  // *  Google Container Registry path. For example:
  //    `gcr.io/projectId/imageName:tag`.
  //
  // *  Artifact Registry path. For example:
  //    `us-central1-docker.pkg.dev/projectId/repoName/imageName:tag`.
  //
  // If a tag is not specified, "latest" will be used as the default tag.
  string output_uri = 1 [(google.api.field_behavior) = REQUIRED];
}
