// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.gkeconnect.gateway.v1;

import "google/api/annotations.proto";
import "google/api/client.proto";
import "google/api/httpbody.proto";

option csharp_namespace = "Google.Cloud.GkeConnect.Gateway.V1";
option go_package = "google.golang.org/genproto/googleapis/cloud/gkeconnect/gateway/v1;gateway";
option java_multiple_files = true;
option java_outer_classname = "GatewayProto";
option java_package = "com.google.cloud.gkeconnect.gateway.v1";
option php_namespace = "Google\\Cloud\\GkeConnect\\Gateway\\V1";
option ruby_package = "Google::Cloud::GkeConnect::Gateway::V1";

// Gateway service is a public API which works as a Kubernetes resource model
// proxy between end users and registered Kubernetes clusters. Each RPC in this
// service matches with an HTTP verb. End user will initiate kubectl commands
// against the Gateway service, and Gateway service will forward user requests
// to clusters.
service GatewayService {
  option (google.api.default_host) = "connectgateway.googleapis.com";
}
