// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.bigquery.storage.v1beta1;

option go_package = "google.golang.org/genproto/googleapis/cloud/bigquery/storage/v1beta1;storage";
option java_outer_classname = "AvroProto";
option java_package = "com.google.cloud.bigquery.storage.v1beta1";

// Avro schema.
message AvroSchema {
  // Json serialized schema, as described at
  // https://avro.apache.org/docs/1.8.1/spec.html
  string schema = 1;
}

// Avro rows.
message AvroRows {
  // Binary serialized rows in a block.
  bytes serialized_binary_rows = 1;

  // The count of rows in the returning block.
  int64 row_count = 2;
}
