// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v8.enums;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V8.Enums";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v8/enums;enums";
option java_multiple_files = true;
option java_outer_classname = "MonthOfYearProto";
option java_package = "com.google.ads.googleads.v8.enums";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V8\\Enums";
option ruby_package = "Google::Ads::GoogleAds::V8::Enums";

// Proto file describing days of week.

// Container for enumeration of months of the year, e.g., "January".
message MonthOfYearEnum {
  // Enumerates months of the year, e.g., "January".
  enum MonthOfYear {
    // Not specified.
    UNSPECIFIED = 0;

    // The value is unknown in this version.
    UNKNOWN = 1;

    // January.
    JANUARY = 2;

    // February.
    FEBRUARY = 3;

    // March.
    MARCH = 4;

    // April.
    APRIL = 5;

    // May.
    MAY = 6;

    // June.
    JUNE = 7;

    // July.
    JULY = 8;

    // August.
    AUGUST = 9;

    // September.
    SEPTEMBER = 10;

    // October.
    OCTOBER = 11;

    // November.
    NOVEMBER = 12;

    // December.
    DECEMBER = 13;
  }


}
