// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v8.services;

import "google/ads/googleads/v8/resources/feed_item_set.proto";
import "google/api/annotations.proto";
import "google/api/client.proto";
import "google/api/field_behavior.proto";
import "google/api/resource.proto";
import "google/protobuf/field_mask.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V8.Services";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v8/services;services";
option java_multiple_files = true;
option java_outer_classname = "FeedItemSetServiceProto";
option java_package = "com.google.ads.googleads.v8.services";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V8\\Services";
option ruby_package = "Google::Ads::GoogleAds::V8::Services";

// Proto file describing the FeedItemSet service.

// Service to manage feed Item Set
service FeedItemSetService {
  option (google.api.default_host) = "googleads.googleapis.com";
  option (google.api.oauth_scopes) = "https://www.googleapis.com/auth/adwords";

  // Returns the requested feed item set in full detail.
  //
  // List of thrown errors:
  //   [AuthenticationError]()
  //   [AuthorizationError]()
  //   [HeaderError]()
  //   [InternalError]()
  //   [QuotaError]()
  //   [RequestError]()
  rpc GetFeedItemSet(GetFeedItemSetRequest) returns (google.ads.googleads.v8.resources.FeedItemSet) {
    option (google.api.http) = {
      get: "/v8/{resource_name=customers/*/feedItemSets/*}"
    };
    option (google.api.method_signature) = "resource_name";
  }

  // Creates, updates or removes feed item sets. Operation statuses are
  // returned.
  //
  // List of thrown errors:
  //   [AuthenticationError]()
  //   [AuthorizationError]()
  //   [HeaderError]()
  //   [InternalError]()
  //   [MutateError]()
  //   [QuotaError]()
  //   [RequestError]()
  rpc MutateFeedItemSets(MutateFeedItemSetsRequest) returns (MutateFeedItemSetsResponse) {
    option (google.api.http) = {
      post: "/v8/customers/{customer_id=*}/feedItemSets:mutate"
      body: "*"
    };
    option (google.api.method_signature) = "customer_id,operations";
  }
}

// Request message for [FeedItemSetService.GetFeedItemSet][google.ads.googleads.v8.services.FeedItemSetService.GetFeedItemSet].
message GetFeedItemSetRequest {
  // Required. The resource name of the feed item set to fetch.
  string resource_name = 1 [
    (google.api.field_behavior) = REQUIRED,
    (google.api.resource_reference) = {
      type: "googleads.googleapis.com/FeedItemSet"
    }
  ];
}

// Request message for [FeedItemSetService.MutateFeedItemSets][google.ads.googleads.v8.services.FeedItemSetService.MutateFeedItemSets].
message MutateFeedItemSetsRequest {
  // Required. The ID of the customer whose feed item sets are being modified.
  string customer_id = 1 [(google.api.field_behavior) = REQUIRED];

  // Required. The list of operations to perform on individual feed item sets.
  repeated FeedItemSetOperation operations = 2 [(google.api.field_behavior) = REQUIRED];

  // If true, successful operations will be carried out and invalid
  // operations will return errors. If false, all operations will be carried
  // out in one transaction if and only if they are all valid.
  // Default is false.
  bool partial_failure = 3;

  // If true, the request is validated but not executed. Only errors are
  // returned, not results.
  bool validate_only = 4;
}

// A single operation (create, remove) on an feed item set.
message FeedItemSetOperation {
  // FieldMask that determines which resource fields are modified in an update.
  google.protobuf.FieldMask update_mask = 4;

  // The mutate operation.
  oneof operation {
    // Create operation: No resource name is expected for the new feed item set
    google.ads.googleads.v8.resources.FeedItemSet create = 1;

    // Update operation: The feed item set is expected to have a valid resource
    // name.
    google.ads.googleads.v8.resources.FeedItemSet update = 2;

    // Remove operation: A resource name for the removed feed item is
    // expected, in this format:
    // `customers/{customer_id}/feedItems/{feed_id}~{feed_item_set_id}`
    string remove = 3;
  }
}

// Response message for an feed item set mutate.
message MutateFeedItemSetsResponse {
  // All results for the mutate.
  repeated MutateFeedItemSetResult results = 1;
}

// The result for the feed item set mutate.
message MutateFeedItemSetResult {
  // Returned for successful operations.
  string resource_name = 1;
}
