// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v8.common;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V8.Common";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v8/common;common";
option java_multiple_files = true;
option java_outer_classname = "ValueProto";
option java_package = "com.google.ads.googleads.v8.common";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V8\\Common";
option ruby_package = "Google::Ads::GoogleAds::V8::Common";

// Proto file describing value types

// A generic data container.
message Value {
  // A value.
  oneof value {
    // A boolean.
    bool boolean_value = 1;

    // An int64.
    int64 int64_value = 2;

    // A float.
    float float_value = 3;

    // A double.
    double double_value = 4;

    // A string.
    string string_value = 5;
  }
}
