// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.talent.v4;

import "google/api/field_behavior.proto";
import "google/api/resource.proto";
import "google/api/annotations.proto";

option go_package = "google.golang.org/genproto/googleapis/cloud/talent/v4;talent";
option java_multiple_files = true;
option java_outer_classname = "TenantProto";
option java_package = "com.google.cloud.talent.v4";
option objc_class_prefix = "CTS";

// A Tenant resource represents a tenant in the service. A tenant is a group or
// entity that shares common access with specific privileges for resources like
// jobs. Customer may create multiple tenants to provide data isolation for
// different groups.
message Tenant {
  option (google.api.resource) = {
    type: "jobs.googleapis.com/Tenant"
    pattern: "projects/{project}/tenants/{tenant}"
  };

  // Required during tenant update.
  //
  // The resource name for a tenant. This is generated by the service when a
  // tenant is created.
  //
  // The format is "projects/{project_id}/tenants/{tenant_id}", for example,
  // "projects/foo/tenants/bar".
  string name = 1;

  // Required. Client side tenant identifier, used to uniquely identify the tenant.
  //
  // The maximum number of allowed characters is 255.
  string external_id = 2 [(google.api.field_behavior) = REQUIRED];
}
