// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v8.resources;

import "google/api/field_behavior.proto";
import "google/api/resource.proto";
import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V8.Resources";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v8/resources;resources";
option java_multiple_files = true;
option java_outer_classname = "SmartCampaignSettingProto";
option java_package = "com.google.ads.googleads.v8.resources";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V8\\Resources";
option ruby_package = "Google::Ads::GoogleAds::V8::Resources";

// Proto file describing the Smart campaign setting resource.

// Settings for configuring Smart campaigns.
message SmartCampaignSetting {
  option (google.api.resource) = {
    type: "googleads.googleapis.com/SmartCampaignSetting"
    pattern: "customers/{customer_id}/smartCampaignSettings/{campaign_id}"
  };

  // Phone number and country code in smart campaign settings.
  message PhoneNumber {
    // Phone number of the smart campaign.
    optional string phone_number = 1;

    // Upper-case, two-letter country code as defined by ISO-3166.
    optional string country_code = 2;
  }

  // Immutable. The resource name of the Smart campaign setting.
  // Smart campaign setting resource names have the form:
  //
  // `customers/{customer_id}/smartCampaignSettings/{campaign_id}`
  string resource_name = 1 [
    (google.api.field_behavior) = IMMUTABLE,
    (google.api.resource_reference) = {
      type: "googleads.googleapis.com/SmartCampaignSetting"
    }
  ];

  // Output only. The campaign to which these settings apply.
  string campaign = 2 [
    (google.api.field_behavior) = OUTPUT_ONLY,
    (google.api.resource_reference) = {
      type: "googleads.googleapis.com/Campaign"
    }
  ];

  // Phone number and country code.
  PhoneNumber phone_number = 3;

  // Landing page url given by user for this Campaign.
  string final_url = 4;

  // The ISO-639-1 language code to advertise in.
  string advertising_language_code = 7;

  // The business setting of this campaign.
  oneof business_setting {
    // The name of the business.
    string business_name = 5;

    // The ID of the Google My Business (GMB) Location.
    // The location ID can be fetched by GMB API with its form:
    // accounts/{accountId}/locations/{locationId}. The last {locationId}
    // component from the GMB API represents the business_location_id.
    // See the [Google My Business API]
    // (https://developers.google.com/my-business/reference/rest/v4/accounts.locations)
    int64 business_location_id = 6;
  }
}
