// Copyright 2020 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.actions.sdk.v2.interactionmodel.prompt;

import "google/actions/sdk/v2/interactionmodel/prompt/content/static_image_prompt.proto";
import "google/api/field_behavior.proto";

option go_package = "google.golang.org/genproto/googleapis/actions/sdk/v2/interactionmodel/prompt;prompt";
option java_multiple_files = true;
option java_outer_classname = "StaticCollectionPromptProto";
option java_package = "com.google.actions.sdk.v2.interactionmodel.prompt";

// A card for presenting a collection of options to select from.
message StaticCollectionPrompt {
  // An item in the collection.
  message CollectionItem {
    // Required. The NLU key that matches the entry key name in the associated
    // Type. When item tapped, this key will be posted back as a select option
    // parameter.
    string key = 1 [(google.api.field_behavior) = REQUIRED];

    // Required. Title of the item. When tapped, this text will be
    // posted back to the conversation verbatim as if the user had typed it.
    // Each title must be unique among the set of items.
    string title = 2 [(google.api.field_behavior) = REQUIRED];

    // Optional. Body text of the item.
    string description = 3 [(google.api.field_behavior) = OPTIONAL];

    // Optional. Item image.
    StaticImagePrompt image = 4 [(google.api.field_behavior) = OPTIONAL];
  }

  // Optional. Title of the collection.
  string title = 1 [(google.api.field_behavior) = OPTIONAL];

  // Optional. Subtitle of the collection.
  string subtitle = 2 [(google.api.field_behavior) = OPTIONAL];

  // Required. Collection items.
  repeated CollectionItem items = 3 [(google.api.field_behavior) = REQUIRED];

  // Optional. Type of image display option.
  StaticImagePrompt.ImageFill image_fill = 4 [(google.api.field_behavior) = OPTIONAL];
}
