// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v6.enums;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V6.Enums";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v6/enums;enums";
option java_multiple_files = true;
option java_outer_classname = "ExtensionTypeProto";
option java_package = "com.google.ads.googleads.v6.enums";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V6\\Enums";
option ruby_package = "Google::Ads::GoogleAds::V6::Enums";

// Proto file describing extension type.

// Container for enum describing possible data types for an extension in an
// extension setting.
message ExtensionTypeEnum {
  // Possible data types for an extension in an extension setting.
  enum ExtensionType {
    // Not specified.
    UNSPECIFIED = 0;

    // Used for return value only. Represents value unknown in this version.
    UNKNOWN = 1;

    // None.
    NONE = 2;

    // App.
    APP = 3;

    // Call.
    CALL = 4;

    // Callout.
    CALLOUT = 5;

    // Message.
    MESSAGE = 6;

    // Price.
    PRICE = 7;

    // Promotion.
    PROMOTION = 8;

    // Sitelink.
    SITELINK = 10;

    // Structured snippet.
    STRUCTURED_SNIPPET = 11;

    // Location.
    LOCATION = 12;

    // Affiliate location.
    AFFILIATE_LOCATION = 13;

    // Hotel callout
    HOTEL_CALLOUT = 15;

    // Image.
    IMAGE = 16;
  }


}
