// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.aiplatform.v1beta1.schema.predict.prediction;

import "google/api/annotations.proto";

option go_package = "google.golang.org/genproto/googleapis/cloud/aiplatform/v1beta1/schema/predict/prediction;prediction";
option java_multiple_files = true;
option java_outer_classname = "TextSentimentPredictionResultProto";
option java_package = "com.google.cloud.aiplatform.v1beta1.schema.predict.prediction";

// Prediction output format for Text Sentiment
message TextSentimentPredictionResult {
  // The integer sentiment labels between 0 (inclusive) and sentimentMax label
  // (inclusive), while 0 maps to the least positive sentiment and
  // sentimentMax maps to the most positive one. The higher the score is, the
  // more positive the sentiment in the text snippet is. Note: sentimentMax is
  // an integer value between 1 (inclusive) and 10 (inclusive).
  int32 sentiment = 1;
}
