// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v7.enums;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V7.Enums";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v7/enums;enums";
option java_multiple_files = true;
option java_outer_classname = "CampaignExperimentTypeProto";
option java_package = "com.google.ads.googleads.v7.enums";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V7\\Enums";
option ruby_package = "Google::Ads::GoogleAds::V7::Enums";

// Proto file describing campaign experiment type.

// Container for enum describing campaign experiment type.
message CampaignExperimentTypeEnum {
  // Indicates if this campaign is a normal campaign,
  // a draft campaign, or an experiment campaign.
  enum CampaignExperimentType {
    // Not specified.
    UNSPECIFIED = 0;

    // Used for return value only. Represents value unknown in this version.
    UNKNOWN = 1;

    // This is a regular campaign.
    BASE = 2;

    // This is a draft version of a campaign.
    // It has some modifications from a base campaign,
    // but it does not serve or accrue metrics.
    DRAFT = 3;

    // This is an experiment version of a campaign.
    // It has some modifications from a base campaign,
    // and a percentage of traffic is being diverted
    // from the BASE campaign to this experiment campaign.
    EXPERIMENT = 4;
  }


}
