// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.cloud.aiplatform.v1.schema.predict.instance;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Cloud.AIPlatform.V1.Schema.Predict.Instance";
option go_package = "google.golang.org/genproto/googleapis/cloud/aiplatform/v1/schema/predict/instance;instance";
option java_multiple_files = true;
option java_outer_classname = "TextSentimentPredictionInstanceProto";
option java_package = "com.google.cloud.aiplatform.v1.schema.predict.instance";
option php_namespace = "Google\\Cloud\\AIPlatform\\V1\\Schema\\Predict\\Instance";
option ruby_package = "Google::Cloud::AIPlatform::V1::Schema::Predict::Instance";

// Prediction input format for Text Sentiment.
message TextSentimentPredictionInstance {
  // The text snippet to make the predictions on.
  string content = 1;

  // The MIME type of the text snippet. The supported MIME types are listed
  // below.
  // - text/plain
  string mime_type = 2;
}
