// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v7.enums;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V7.Enums";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v7/enums;enums";
option java_multiple_files = true;
option java_outer_classname = "ChangeEventResourceTypeProto";
option java_package = "com.google.ads.googleads.v7.enums";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V7\\Enums";
option ruby_package = "Google::Ads::GoogleAds::V7::Enums";

// Proto file describing the resource types the ChangeEvent resource supports.

// Container for enum describing supported resource types for the ChangeEvent
// resource.
message ChangeEventResourceTypeEnum {
  // Enum listing the resource types support by the ChangeEvent resource.
  enum ChangeEventResourceType {
    // No value has been specified.
    UNSPECIFIED = 0;

    // Used for return value only. Represents an unclassified resource unknown
    // in this version.
    UNKNOWN = 1;

    // An Ad resource change.
    AD = 2;

    // An AdGroup resource change.
    AD_GROUP = 3;

    // An AdGroupCriterion resource change.
    AD_GROUP_CRITERION = 4;

    // A Campaign resource change.
    CAMPAIGN = 5;

    // A CampaignBudget resource change.
    CAMPAIGN_BUDGET = 6;

    // An AdGroupBidModifier resource change.
    AD_GROUP_BID_MODIFIER = 7;

    // A CampaignCriterion resource change.
    CAMPAIGN_CRITERION = 8;

    // A Feed resource change.
    FEED = 9;

    // A FeedItem resource change.
    FEED_ITEM = 10;

    // A CampaignFeed resource change.
    CAMPAIGN_FEED = 11;

    // An AdGroupFeed resource change.
    AD_GROUP_FEED = 12;

    // An AdGroupAd resource change.
    AD_GROUP_AD = 13;
  }


}
