// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.monitoring.dashboard.v1;

import "google/api/field_behavior.proto";
import "google/monitoring/dashboard/v1/alertchart.proto";
import "google/monitoring/dashboard/v1/scorecard.proto";
import "google/monitoring/dashboard/v1/text.proto";
import "google/monitoring/dashboard/v1/xychart.proto";
import "google/protobuf/empty.proto";

option go_package = "google.golang.org/genproto/googleapis/monitoring/dashboard/v1;dashboard";
option java_multiple_files = true;
option java_outer_classname = "WidgetProto";
option java_package = "com.google.monitoring.dashboard.v1";
option php_namespace = "Google\\Cloud\\Monitoring\\Dashboard\\V1";
option ruby_package = "Google::Cloud::Monitoring::Dashboard::V1";

// Widget contains a single dashboard component and configuration of how to
// present the component in the dashboard.
message Widget {
  // Optional. The title of the widget.
  string title = 1 [(google.api.field_behavior) = OPTIONAL];

  // Content defines the component used to populate the widget.
  oneof content {
    // A chart of time series data.
    XyChart xy_chart = 2;

    // A scorecard summarizing time series data.
    Scorecard scorecard = 3;

    // A raw string or markdown displaying textual content.
    Text text = 4;

    // A blank space.
    google.protobuf.Empty blank = 5;

    // A chart of alert policy data.
    AlertChart alert_chart = 7;
  }
}
