// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v6.services;

import "google/ads/googleads/v6/resources/mobile_device_constant.proto";
import "google/api/annotations.proto";
import "google/api/client.proto";
import "google/api/field_behavior.proto";
import "google/api/resource.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V6.Services";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v6/services;services";
option java_multiple_files = true;
option java_outer_classname = "MobileDeviceConstantServiceProto";
option java_package = "com.google.ads.googleads.v6.services";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V6\\Services";
option ruby_package = "Google::Ads::GoogleAds::V6::Services";

// Proto file describing the mobile device constant service.

// Service to fetch mobile device constants.
service MobileDeviceConstantService {
  option (google.api.default_host) = "googleads.googleapis.com";
  option (google.api.oauth_scopes) = "https://www.googleapis.com/auth/adwords";

  // Returns the requested mobile device constant in full detail.
  rpc GetMobileDeviceConstant(GetMobileDeviceConstantRequest) returns (google.ads.googleads.v6.resources.MobileDeviceConstant) {
    option (google.api.http) = {
      get: "/v6/{resource_name=mobileDeviceConstants/*}"
    };
    option (google.api.method_signature) = "resource_name";
  }
}

// Request message for [MobileDeviceConstantService.GetMobileDeviceConstant][google.ads.googleads.v6.services.MobileDeviceConstantService.GetMobileDeviceConstant].
message GetMobileDeviceConstantRequest {
  // Required. Resource name of the mobile device to fetch.
  string resource_name = 1 [
    (google.api.field_behavior) = REQUIRED,
    (google.api.resource_reference) = {
      type: "googleads.googleapis.com/MobileDeviceConstant"
    }
  ];
}
