// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v7.enums;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V7.Enums";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v7/enums;enums";
option java_multiple_files = true;
option java_outer_classname = "LeadFormFieldUserInputTypeProto";
option java_package = "com.google.ads.googleads.v7.enums";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V7\\Enums";
option ruby_package = "Google::Ads::GoogleAds::V7::Enums";

// Describes the input type of a lead form field.
message LeadFormFieldUserInputTypeEnum {
  // Enum describing the input type of a lead form field.
  enum LeadFormFieldUserInputType {
    // Not specified.
    UNSPECIFIED = 0;

    // Used for return value only. Represents value unknown in this version.
    UNKNOWN = 1;

    // The user will be asked to fill in their given and family name. This field
    // cannot be set at the same time as GIVEN_NAME or FAMILY_NAME.
    FULL_NAME = 2;

    // The user will be asked to fill in their given name (first name). This
    // field can not be set at the same time as FULL_NAME.
    GIVEN_NAME = 6;

    // The user will be asked to fill in their family name (last name). This
    // field can not be set at the same time as FULL_NAME.
    FAMILY_NAME = 7;

    // The user will be asked to fill in their email address.
    EMAIL = 3;

    // The user will be asked to fill in their phone number.
    PHONE_NUMBER = 4;

    // The user will be asked to fill in their zip code.
    POSTAL_CODE = 5;

    // The user will be asked to fill in their city.
    CITY = 9;

    // The user will be asked to fill in their region part of the address (e.g.
    // state for US, province for Canada).
    REGION = 10;

    // The user will be asked to fill in their country.
    COUNTRY = 11;

    // The user will be asked to fill in their work email address.
    WORK_EMAIL = 12;

    // The user will be asked to fill in their company name.
    COMPANY_NAME = 13;

    // The user will be asked to fill in their work phone.
    WORK_PHONE = 14;

    // The user will be asked to fill in their job title.
    JOB_TITLE = 15;

    // Question: "Which model are you interested in?"
    // Category: "Auto"
    VEHICLE_MODEL = 1001;

    // Question: "Which type of vehicle are you interested in?"
    // Category: "Auto"
    VEHICLE_TYPE = 1002;

    // Question: "What is your preferred dealership?"
    // Category: "Auto"
    PREFERRED_DEALERSHIP = 1003;

    // Question: "When do you plan on purchasing a vehicle?"
    // Category: "Auto"
    VEHICLE_PURCHASE_TIMELINE = 1004;

    // Question: "Do you own a vehicle?"
    // Category: "Auto"
    VEHICLE_OWNERSHIP = 1005;

    // Question: "What vehicle ownership option are you interested in?"
    // Category: "Auto"
    VEHICLE_PAYMENT_TYPE = 1009;

    // Question: "What type of vehicle condition are you interested in?"
    // Category: "Auto"
    VEHICLE_CONDITION = 1010;

    // Question: "What size is your company?"
    // Category: "Business"
    COMPANY_SIZE = 1006;

    // Question: "What is your annual sales volume?"
    // Category: "Business"
    ANNUAL_SALES = 1007;

    // Question: "How many years have you been in business?"
    // Category: "Business"
    YEARS_IN_BUSINESS = 1008;

    // Question: "What is your job department?"
    // Category: "Business"
    JOB_DEPARTMENT = 1011;

    // Question: "What is your job role?"
    // Category: "Business"
    JOB_ROLE = 1012;

    // Question: "Which program are you interested in?"
    // Category: "Education"
    EDUCATION_PROGRAM = 1013;

    // Question: "Which course are you interested in?"
    // Category: "Education"
    EDUCATION_COURSE = 1014;

    // Question: "Which product are you interested in?"
    // Category: "General"
    PRODUCT = 1016;

    // Question: "Which service are you interested in?"
    // Category: "General"
    SERVICE = 1017;

    // Question: "Which offer are you interested in?"
    // Category: "General"
    OFFER = 1018;

    // Question: "Which category are you interested in?"
    // Category: "General"
    CATEGORY = 1019;

    // Question: "What is your preferred method of contact?"
    // Category: "General"
    PREFERRED_CONTACT_METHOD = 1020;

    // Question: "What is your preferred location?"
    // Category: "General"
    PREFERRED_LOCATION = 1021;

    // Question: "What is the best time to contact you?"
    // Category: "General"
    PREFERRED_CONTACT_TIME = 1022;

    // Question: "When are you looking to make a purchase?"
    // Category: "General"
    PURCHASE_TIMELINE = 1023;

    // Question: "How many years of work experience do you have?"
    // Category: "Jobs"
    YEARS_OF_EXPERIENCE = 1048;

    // Question: "What industry do you work in?"
    // Category: "Jobs"
    JOB_INDUSTRY = 1049;

    // Question: "What is your highest level of education?"
    // Category: "Jobs"
    LEVEL_OF_EDUCATION = 1050;

    // Question: "What type of property are you looking for?"
    // Category: "Real Estate"
    PROPERTY_TYPE = 1024;

    // Question: "What do you need a realtor's help with?"
    // Category: "Real Estate"
    REALTOR_HELP_GOAL = 1025;

    // Question: "What neighborhood are you interested in?"
    // Category: "Real Estate"
    PROPERTY_COMMUNITY = 1026;

    // Question: "What price range are you looking for?"
    // Category: "Real Estate"
    PRICE_RANGE = 1027;

    // Question: "How many bedrooms are you looking for?"
    // Category: "Real Estate"
    NUMBER_OF_BEDROOMS = 1028;

    // Question: "Are you looking for a fully furnished property?"
    // Category: "Real Estate"
    FURNISHED_PROPERTY = 1029;

    // Question: "Are you looking for properties that allow pets?"
    // Category: "Real Estate"
    PETS_ALLOWED_PROPERTY = 1030;

    // Question: "What is the next product you plan to purchase?"
    // Category: "Retail"
    NEXT_PLANNED_PURCHASE = 1031;

    // Question: "Would you like to sign up for an event?"
    // Category: "Retail"
    EVENT_SIGNUP_INTEREST = 1033;

    // Question: "Where are you interested in shopping?"
    // Category: "Retail"
    PREFERRED_SHOPPING_PLACES = 1034;

    // Question: "What is your favorite brand?"
    // Category: "Retail"
    FAVORITE_BRAND = 1035;

    // Question: "Which type of valid commercial license do you have?"
    // Category: "Transportation"
    TRANSPORTATION_COMMERCIAL_LICENSE_TYPE = 1036;

    // Question: "Interested in booking an event?"
    // Category: "Travel"
    EVENT_BOOKING_INTEREST = 1038;

    // Question: "What is your destination country?"
    // Category: "Travel"
    DESTINATION_COUNTRY = 1039;

    // Question: "What is your destination city?"
    // Category: "Travel"
    DESTINATION_CITY = 1040;

    // Question: "What is your departure country?"
    // Category: "Travel"
    DEPARTURE_COUNTRY = 1041;

    // Question: "What is your departure city?"
    // Category: "Travel"
    DEPARTURE_CITY = 1042;

    // Question: "What is your departure date?"
    // Category: "Travel"
    DEPARTURE_DATE = 1043;

    // Question: "What is your return date?"
    // Category: "Travel"
    RETURN_DATE = 1044;

    // Question: "How many people are you traveling with?"
    // Category: "Travel"
    NUMBER_OF_TRAVELERS = 1045;

    // Question: "What is your travel budget?"
    // Category: "Travel"
    TRAVEL_BUDGET = 1046;

    // Question: "Where do you want to stay during your travel?"
    // Category: "Travel"
    TRAVEL_ACCOMMODATION = 1047;
  }


}
