// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package maps.fleetengine.v1;

import "google/api/field_behavior.proto";

option go_package = "google.golang.org/genproto/googleapis/maps/fleetengine/v1;fleetengine";
option java_multiple_files = true;
option java_outer_classname = "Headers";
option java_package = "google.maps.fleetengine.v1";
option objc_class_prefix = "CFE";

// A RequestHeader contains fields common to all Fleet Engine RPC requests.
message RequestHeader {
  // The language requested. The external form of Google International
  // Identifiers Initiative (III) LanguageCode objects. If none is specified,
  // return a name in any language, with a preference for English if such a
  // name exists.
  string language_code = 1;

  // Required. CLDR region code of the region where the request originates.
  string region_code = 2 [(google.api.field_behavior) = REQUIRED];

  // Version of the calling SDK, if applicable.
  string sdk_version = 3;

  // Version of the operating system on which the calling SDK is running.
  string os_version = 4;

  // Model of the device on which the calling SDK is running.
  string device_model = 5;
}
