// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v6.enums;

import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V6.Enums";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v6/enums;enums";
option java_multiple_files = true;
option java_outer_classname = "FeedItemQualityApprovalStatusProto";
option java_package = "com.google.ads.googleads.v6.enums";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V6\\Enums";
option ruby_package = "Google::Ads::GoogleAds::V6::Enums";

// Proto file describing feed item quality evaluation approval statuses.

// Container for enum describing possible quality evaluation approval statuses
// of a feed item.
message FeedItemQualityApprovalStatusEnum {
  // The possible quality evaluation approval statuses of a feed item.
  enum FeedItemQualityApprovalStatus {
    // No value has been specified.
    UNSPECIFIED = 0;

    // Used for return value only. Represents value unknown in this version.
    UNKNOWN = 1;

    // Meets all quality expectations.
    APPROVED = 2;

    // Does not meet some quality expectations. The specific reason is found in
    // the quality_disapproval_reasons field.
    DISAPPROVED = 3;
  }


}
