// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v8.resources;

import "google/ads/googleads/v8/enums/customer_pay_per_conversion_eligibility_failure_reason.proto";
import "google/api/field_behavior.proto";
import "google/api/resource.proto";
import "google/api/annotations.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V8.Resources";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v8/resources;resources";
option java_multiple_files = true;
option java_outer_classname = "CustomerProto";
option java_package = "com.google.ads.googleads.v8.resources";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V8\\Resources";
option ruby_package = "Google::Ads::GoogleAds::V8::Resources";

// Proto file describing the Customer resource.

// A customer.
message Customer {
  option (google.api.resource) = {
    type: "googleads.googleapis.com/Customer"
    pattern: "customers/{customer_id}"
  };

  // Immutable. The resource name of the customer.
  // Customer resource names have the form:
  //
  // `customers/{customer_id}`
  string resource_name = 1 [
    (google.api.field_behavior) = IMMUTABLE,
    (google.api.resource_reference) = {
      type: "googleads.googleapis.com/Customer"
    }
  ];

  // Output only. The ID of the customer.
  optional int64 id = 19 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Optional, non-unique descriptive name of the customer.
  optional string descriptive_name = 20;

  // Immutable. The currency in which the account operates.
  // A subset of the currency codes from the ISO 4217 standard is
  // supported.
  optional string currency_code = 21 [(google.api.field_behavior) = IMMUTABLE];

  // Immutable. The local timezone ID of the customer.
  optional string time_zone = 22 [(google.api.field_behavior) = IMMUTABLE];

  // The URL template for constructing a tracking URL out of parameters.
  optional string tracking_url_template = 23;

  // The URL template for appending params to the final URL
  optional string final_url_suffix = 24;

  // Whether auto-tagging is enabled for the customer.
  optional bool auto_tagging_enabled = 25;

  // Output only. Whether the Customer has a Partners program badge. If the Customer is not
  // associated with the Partners program, this will be false. For more
  // information, see https://support.google.com/partners/answer/3125774.
  optional bool has_partners_badge = 26 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. Whether the customer is a manager.
  optional bool manager = 27 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. Whether the customer is a test account.
  optional bool test_account = 28 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Call reporting setting for a customer.
  CallReportingSetting call_reporting_setting = 10;

  // Output only. Conversion tracking setting for a customer.
  ConversionTrackingSetting conversion_tracking_setting = 14 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. Remarketing setting for a customer.
  RemarketingSetting remarketing_setting = 15 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. Reasons why the customer is not eligible to use PaymentMode.CONVERSIONS. If
  // the list is empty, the customer is eligible. This field is read-only.
  repeated google.ads.googleads.v8.enums.CustomerPayPerConversionEligibilityFailureReasonEnum.CustomerPayPerConversionEligibilityFailureReason pay_per_conversion_eligibility_failure_reasons = 16 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. Optimization score of the customer.
  //
  // Optimization score is an estimate of how well a customer's campaigns are
  // set to perform. It ranges from 0% (0.0) to 100% (1.0). This field is null
  // for all manager customers, and for unscored non-manager customers.
  //
  // See "About optimization score" at
  // https://support.google.com/google-ads/answer/9061546.
  //
  // This field is read-only.
  optional double optimization_score = 29 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. Optimization score weight of the customer.
  //
  // Optimization score weight can be used to compare/aggregate optimization
  // scores across multiple non-manager customers. The aggregate optimization
  // score of a manager is computed as the sum over all of their customers of
  // `Customer.optimization_score * Customer.optimization_score_weight`. This
  // field is 0 for all manager customers, and for unscored non-manager
  // customers.
  //
  // This field is read-only.
  double optimization_score_weight = 30 [(google.api.field_behavior) = OUTPUT_ONLY];
}

// Call reporting setting for a customer.
message CallReportingSetting {
  // Enable reporting of phone call events by redirecting them via Google
  // System.
  optional bool call_reporting_enabled = 10;

  // Whether to enable call conversion reporting.
  optional bool call_conversion_reporting_enabled = 11;

  // Customer-level call conversion action to attribute a call conversion to.
  // If not set a default conversion action is used. Only in effect when
  // call_conversion_reporting_enabled is set to true.
  optional string call_conversion_action = 12 [(google.api.resource_reference) = {
                                                 type: "googleads.googleapis.com/ConversionAction"
                                               }];
}

// A collection of customer-wide settings related to Google Ads Conversion
// Tracking.
message ConversionTrackingSetting {
  // Output only. The conversion tracking id used for this account. This id is automatically
  // assigned after any conversion tracking feature is used. If the customer
  // doesn't use conversion tracking, this is 0. This field is read-only.
  optional int64 conversion_tracking_id = 3 [(google.api.field_behavior) = OUTPUT_ONLY];

  // Output only. The conversion tracking id of the customer's manager. This is set when the
  // customer is opted into cross account conversion tracking, and it overrides
  // conversion_tracking_id. This field can only be managed through the Google
  // Ads UI. This field is read-only.
  optional int64 cross_account_conversion_tracking_id = 4 [(google.api.field_behavior) = OUTPUT_ONLY];
}

// Remarketing setting for a customer.
message RemarketingSetting {
  // Output only. The Google global site tag.
  optional string google_global_site_tag = 2 [(google.api.field_behavior) = OUTPUT_ONLY];
}
